;;; Phone.lsp   Ver. 0.7
;;; 1-1-95
;;; David A. Seibert
;;;
;;; Program to track contacts inside AutoCAD
;;;

(defun c:phone(/ Screen)                        ;begin main
  (setq Screen
    (list "TContact"  "TCompany"  "TAddress1"
          "TAddress2" "TCity"     "TState"
          "TZipcode"  "TPhone"    "TFax"
          "TNote1"    "TNote2"
    )
  )                                             ;screen structure
  (set_defaults)                                ;set defaults
  (setq num (load_dialog "phon07"))             ;set number for dialog box
  (new_dialog "phon07" num)                     ;invoke new dialog box
  (mode_tile "TContact" 2)                      ;set initial focus to company
  (set_tile "error"
    (strcat DirPath "/" FileName ".dat"))       ;show current datafile
  (action_tile "TSearch" "(search_record)")     ;action for search button
  (action_tile "TNext" "(next_record)")         ;action for next button
  (action_tile "TPrevious" "(prev_record)")     ;action for next button
  (action_tile "TUpdate" "(update_record)")     ;action for update button
  (action_tile "TDelete" "(delete_record)")     ;action for delete button
  (action_tile "TOptions" "(option_dialog)")    ;action for options button
  (action_tile "THelp" "(help_dialog)")         ;action for help button
  (action_tile "cancel" "(done_dialog)")        ;action for cancel but
  (start_dialog)                                ;start dialog
  (unload_dialog num)                           ;unload dialog
  (princ)                                       ;clean exit
)                                               ;end main

;;;set_defaults function
;;;It will read the default settings
;;;from the ".ini" file and set the
;;;operating variables accordingly

(defun set_defaults(/ InFile LineIn)            ;begin set_defaults
  (open_infile "PHON07" ".ini")
  (while                                        ;while
    (setq LineIn (read-line InFile))            ;there are lines to read
    (if (= (substr LineIn 1 10) "directory=")   ;if the line read is the
      (setq DirPath (substr LineIn 11 100))     ;directory setting then
                                                ;set the directory variable
                                                ;to this value
      (if (= (substr LineIn 1 9) "filename=")   ;if the line read is the
        (setq FileName (substr LineIn 10 18))   ;filename setting then
                                                ;set the filename variable
                                                ;to this value
      )                                         ;end if
    )                                           ;end if
  )                                             ;end while
  (close InFile)                                ;close ".ini" file
)

;;;option_dialog function
;;;this function puts option dialog box
;;;on screen

(defun option_dialog ()                         ;Begin option_dialog
  (new_dialog "option" num)                     ;set number for dialog box
  (set_tile "TDirectory" DirPath)               ;set dialog directory path
                                                ;line to DirPath variable
  (set_tile "TFilName" FileName)                ;set dialog Directory file
                                                ;line to FileName variable
  (mode_tile "TDirectory" 2)                    ;set focus to directory box
  (action_tile "TOk"
               "(save_options)(done_dialog)")   ;option_action and close dialog
  (action_tile "cancel" "(done_dialog)")        ;make "Cancel" close dialog
  (start_dialog)                                ;start dialog box
)                                               ;End option_dialog

;;;option_action function
;;;This function will rewrite the ".ini" file with the
;;;values set in the dialog box

(defun save_options ()                          ;begin save_options
  (open_outfile "PHON07" ".ini")
  (write-line (strcat                           ;write directory path from
                "directory="                    ;dialog box to ".ini" file
                (get_tile "TDirectory")
              )
              OutFile
  )
  (setq DirPath (get_tile "TDirectory"))        ;reset directory path
  (write-line (strcat                           ;write filename from dialog
                "filename="                     ;box to ".ini" file
                (get_tile "TFilName")
              )
              OutFile
  )
  (setq FileName (get_tile "TFilName"))         ;reset database filename
  (close OutFile)                               ;close ".ini" file
  (if (not (findfile
            (strcat
              (get_tile "TDirectory")
              "/"
              (get_tile "TFilName")
              ".dat"
            )
           )
      )
    (make_data_file DirPath FileName)
  )
  (setq FileName (get_tile "TFilName"))
  (setq DirPath (get_tile "TDirectory"))
  (alert "Options saved")                       ;signal when done
)                                               ;end save_options

;;;search_record function
;;;It will query the database for the
;;;record based on CompanyName or
;;;ContactName
(defun search_record ()                         ;begin search_record
  (setq Check (get_tile "TContact")             ;set check variable
        RecNdx nil
        RecordList nil
        flag FlagStat
        RecCnt 0
        InFile (open
                 (strcat
                   DirPath "/" FileName ".dat"
                 )
                 "r"
               )
  )
  (while (= flag nil)                           ;while flag is nil
    (get_from_file)
    (if (/= Record nil)                         ;if record exists
      (if (or (strstr
                (strcase (nth 1 Record))        ;and
                (strcase Check)                 ;if Company is search string
              )                                 ;end string compare
              (strstr                           ;or
                (strcase (nth 0 Record))        ;and
                (strcase Check)                 ;Contact is search string
              )                                 ;end string compare
          )                                     ;close or
        (setq RecordList
          (cons Record RecordList)
        )
      )                                         ;end if
      (setq flag 1)                             ;set break flag
    )                                           ;end if
    (setq RecCnt (+ RecCnt 1))
  )                                             ;end while
  (close InFile)                                ;close ".dat" file
  (next_record)
)                                               ;end search_record

;;;next_record function
;;;It will query the record list for the
;;;next record
(defun next_record ()                           ;begin next_record
  (if (/= RecordList nil)                       ;if record list exists
    (progn
      (if (= RecNdx nil)                        ;if no record index
        (setq RecNdx 0)                         ;set index to 0
        (setq RecNdx (+ RecNdx 1))              ;else add 1
      )                                         ;end if
      (setq Record (nth RecNdx RecordList))     ;set record to index item
      (if (/= Record nil)                       ;if record exists
          (put_to_screen)                       ;show record
        (progn
          (setq RecNdx (- RecNdx 1))
          (alert "End of records")              ;prompt bad search
        )
      )                                         ;end if
    )
    (alert "Name not found")                    ;inform
  )                                             ;end if
)                                               ;end next_record

;;;prev_record function
;;;It will query the record list for the
;;;previous record
(defun prev_record ()                           ;begin prev_record
  (if (/= RecordList nil)                       ;if record list exists
    (progn
      (setq RecNdx (- RecNdx 1))                ;set index back 1
      (if (and (/= RecordList nil)              ;if record exists
               (>= RecNdx 0))                   ;and index is positive
        (progn
          (setq Record (nth RecNdx RecordList)) ;set record to index item
          (put_to_screen)                       ;show record
        )
        (progn
          (setq RecNdx (+ RecNdx 1))
          (alert "No previous")                 ;prompt bad search
        )
      )                                         ;end if
    )
    (alert "Name not found")                    ;inform
  )                                             ;end if
)                                               ;end prev_record

;;;update_record function
;;;This function will update records in the database

(defun update_record(/ Contact     InFile
                                   OutFile)     ;begin udate_record
  (setq Contact (get_tile "TContact"))          ;set Contact variable
  (backup_file DirPath FileName ".dat")
  (setq flag nil
        InFile (open
                 (strcat
                   DirPath "/" FileName ".bak"
                 )
                 "r"
               )
        OutFile (open
                 (strcat
                   DirPath "/" FileName ".dat"
                 )
                 "w"
               )
  )
  ;;this block writes data to new ".dat" file
  (write-line (get_tile "TContact") OutFile)
  (write-line (get_tile "TCompany") OutFile)
  (write-line (get_tile "TAddress1") OutFile)
  (write-line (get_tile "TAddress2") OutFile)
  (write-line (get_tile "TCity") OutFile)
  (write-line (get_tile "TState") OutFile)
  (write-line (get_tile "TZipcode") OutFile)
  (write-line (get_tile "TPhone") OutFile)
  (write-line (get_tile "TFax") OutFile)
  (write-line (get_tile "TNote1") OutFile)
  (write-line (get_tile "TNote2") OutFile)
  (write-line ";" OutFile)
  ;;this block reads in the old data file
  ;;and writes out all but the updated record
  (while (= flag nil)                           ;while there are still records
    (get_from_file)                             ;retrieve record from file
    (if (/= Record nil)                         ;if record exists
      (if (/= (strcase Contact)
              (strcase (nth 0 Record))          ;if Contact is not equal to
          )                                     ;ContactName
        (put_to_file)                           ;write record to file
      )                                         ;end if
      (setq flag 1)
    )                                           ;end if
  )                                             ;end while
  (alert "Update complete")                     ;signal when done
  (close InFile)                                ;close ".bak" file
  (close OutFile)                               ;close ".dat" file
)                                               ;end update_record

;;;Begin delete_record function
;;;This function will delete records from the database

(defun delete_record (/ Contact
                        InFile      OutFile)    ;begin delete_record
  (setq Contact (get_tile "TContact"))          ;set Contact variable to
  (backup_file DirPath FileName ".dat")
  (setq InFile (open
                 (strcat
                   DirPath "/" FileName ".bak"
                 )
                 "r"
               )
        OutFile (open
                 (strcat
                   DirPath "/" FileName ".dat"
                 )
                 "w"
               )
  )
  (while (/= Record nil)
    (get_from_file)
    (if (/= Record nil)                         ;if more records
      (if (/= (strcase Contact)
              (strcase (nth 0 Record))          ;if Contact is not equal to
          )                                     ;ContactName
        (put_to_file)                           ;write record to file
      )                                         ;end if
    )                                           ;end if
  )                                             ;end while
  (alert "Record Deleted")                      ;signal when done
  (close InFile)
  (close OutFile)
)                                               ;end delete_action

(defun backup_file(DPath DFile FileExt / DPath   DFile  FileExt InFile
                                         OutFile LineIn)
   (setq InFile (open (strcat DPath "/" DFile FileExt) "r"))
   (setq OutFile (open (strcat DPath "/" DFile ".bak") "w"))
   (while
      (setq LineIn (read-line InFile))
      (write-line LineIn OutFile)
   )
   (close InFile)
   (close OutFile)
)
;;;
;;; STRSTR -- Scans a string for the occurrence of a given substring.
;;;           

;;;            If neither argument is a string, *ERRORNO* is set to -1 
;;;            and nil is returned.
;;;
(defun strstr (_$s1 _$s2 / _$j _$sl _$sl2)
  (setq *ERRORNO* nil)
  (if (and (= (type _$s1) 'STR) 
           (= (type _$s2) 'STR)
      )
    (progn
      (setq _$j   0
            _$sl  (strlen _$s1)
            _$sl2 (strlen _$s2)
      )
      (while (< _$j _$sl)
        (if (= (substr _$s1 (setq _$j (1+ _$j)) 1)
               (substr _$s2 1 1)
            )
          (if (= (substr _$s1 _$j _$sl2) _$s2)
            (progn
              (setq _$s1 (substr _$s1 _$j))
              (setq _$j _$sl)
              _$s1
            )
          )
        )
      )
    )
    (progn
      (setq *ERRORNO* -1)
      nil
    )
  )
)

(defun help_dialog()                            ;begin help_dialog
   (new_dialog "help" num)                      ;call help dialog
  (start_list "HELP")                           ;begin listbox list
  (setq InFile (open
                 (findfile
                   (strcat "phon07" ".hlp")
                 )
                 "r"
               ))                               ;open help file for read
  (while (setq LineIn (read-line InFile))       ;while there are lines
    (add_list LineIn)                           ;add a line to the list
  )                                             ;end while
  (close InFile)                                ;close help file
  (end_list)                                    ;end listbox list
   (action_tile "OK" "(done_dialog)")           ;exit dialog
   (start_dialog)                               ;start dialog
)                                               ;end help_dialog

(defun get_from_file (/ RecIn LineIn)           ;get record from file
  (setq RecIn nil)
  (while (and (/= LineIn ";")
    (setq LineIn (read-line InFile)))
    (setq RecIn (cons LineIn RecIn))
  )
  (setq Record (reverse RecIn))                 ;set the record
  (princ)
)

(defun put_to_screen(/ Ndx Val Til)             ;send record to screen
  (setq ndx 0)
  (while (< ndx 11)
    (setq Val (nth Ndx Record))
    (setq Til (nth Ndx Screen))
    (set_tile Til Val)
    (setq Ndx (+ Ndx 1))
  )
)

(defun put_to_file(/ Ndx Field)                 ;write record to file
  (setq Ndx 0)
  (while (< ndx 11)
    (setq Field (nth Ndx Record))
    (write-line Field OutFile)
    (setq Ndx (+ Ndx 1))
  )
  (write-line ";" OutFile)
)

(defun open_infile(FileNam Ext / FileNam Ext)
  (setq InFile (open                            ;open file for read
                 (findfile
                   (strcat FileNam Ext)
                 )
                 "r"
               )
  )
)

(defun open_outfile(FileNam Ext / FileNam Ext)
  (setq OutFile (open                           ;open file for write
                 (findfile
                   (strcat FileNam Ext)
                 )
                 "w"
               )
  )
)

(defun make_data_file (DirPath FileName)
  (setq OutFile (open (strcat DirPath "/" FileName ".dat") "w"))
  (write-line "SeiberTron Engineering Services" OutFile)
  (write-line "David A. Seibert" OutFile)
  (write-line "851 N. Willow Glen Ave." OutFile)
  (write-line "" OutFile)
  (write-line "Tipp City" OutFile)
  (write-line "OH" OutFile)
  (write-line "45371" OutFile)
  (write-line "513-667-6289" OutFile)
  (write-line "" OutFile)
  (write-line "AutoCAD customization" OutFile)
  (write-line "" OutFile)
  (write-line ";" OutFile)
  (close OutFile)
)
(princ)
